@@ -14,6 +14,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import errno
 import os
 import time
 
@@ -24,7 +25,6 @@ import six
 from octavia.amphorae.backends.agent.api_server import util
 from octavia.amphorae.backends.health_daemon import health_sender
 from octavia.amphorae.backends.utils import haproxy_query
-from octavia.i18n import _LI
 
 if six.PY2:
     import Queue as queue
@@ -32,9 +32,6 @@ else:
     import queue
 
 CONF = cfg.CONF
-CONF.import_group('amphora_agent', 'octavia.common.config')
-CONF.import_group('haproxy_amphora', 'octavia.common.config')
-CONF.import_group('health_manager', 'octavia.common.config')
 LOG = logging.getLogger(__name__)
 SEQ = 0
 
@@ -51,18 +48,54 @@ def list_sock_stat_files(hadir=None):
 
 
 def run_sender(cmd_queue):
-    LOG.info(_LI('Health Manager Sender starting.'))
+    LOG.info('Health Manager Sender starting.')
     sender = health_sender.UDPStatusSender()
+
+    keepalived_cfg_path = util.keepalived_cfg_path()
+    keepalived_pid_path = util.keepalived_pid_path()
+
     while True:
-        message = build_stats_message()
-        sender.dosend(message)
+
+        try:
+            # If the keepalived config file is present check
+            # that it is running, otherwise don't send the health
+            # heartbeat
+            if os.path.isfile(keepalived_cfg_path):
+                # Is there a pid file for keepalived?
+                with open(keepalived_pid_path, 'r') as pid_file:
+                    pid = int(pid_file.readline())
+                os.kill(pid, 0)
+
+            message = build_stats_message()
+            sender.dosend(message)
+
+        except IOError as e:
+            # Missing PID file, skip health heartbeat
+            if e.errno == errno.ENOENT:
+                LOG.error('Missing keepalived PID file %s, skipping health '
+                          'heartbeat.', keepalived_pid_path)
+            else:
+                LOG.error('Failed to check keepalived and haproxy status due '
+                          'to exception %s, skipping health heartbeat.', e)
+        except OSError as e:
+            # Keepalived is not running, skip health heartbeat
+            if e.errno == errno.ESRCH:
+                LOG.error('Keepalived is configured but not running, '
+                          'skipping health heartbeat.')
+            else:
+                LOG.error('Failed to check keepalived and haproxy status due '
+                          'to exception %s, skipping health heartbeat.', e)
+        except Exception as e:
+            LOG.error('Failed to check keepalived and haproxy status due to '
+                      'exception %s, skipping health heartbeat.', e)
+
         try:
             cmd = cmd_queue.get_nowait()
             if cmd is 'reload':
-                LOG.info(_LI('Reloading configuration'))
+                LOG.info('Reloading configuration')
                 CONF.reload_config_files()
             elif cmd is 'shutdown':
-                LOG.info(_LI('Health Manager Sender shutting down.'))
+                LOG.info('Health Manager Sender shutting down.')
                 break
         except queue.Empty:
             pass
@@ -82,10 +115,15 @@ def build_stats_message():
            'seq': SEQ, "listeners": {}}
     SEQ += 1
     stat_sock_files = list_sock_stat_files()
-    for listener_id, stat_sock_file in six.iteritems(stat_sock_files):
-        listener_dict = {'pools': {}, 'status': 'DOWN',
-                                      'stats': {'tx': 0, 'rx': 0,
-                                                'conns': 0, 'totconns': 0}}
+    for listener_id, stat_sock_file in stat_sock_files.items():
+        listener_dict = {'pools': {},
+                         'status': 'DOWN',
+                         'stats': {
+                             'tx': 0,
+                             'rx': 0,
+                             'conns': 0,
+                             'totconns': 0,
+                             'ereq': 0}}
         msg['listeners'][listener_id] = listener_dict
         if util.is_listener_running(listener_id):
             (stats, pool_status) = get_stats(stat_sock_file)
@@ -96,8 +134,9 @@ def build_stats_message():
                     listener_dict['stats']['rx'] = int(row['bin'])
                     listener_dict['stats']['conns'] = int(row['scur'])
                     listener_dict['stats']['totconns'] = int(row['stot'])
+                    listener_dict['stats']['ereq'] = int(row['ereq'])
                     listener_dict['status'] = row['status']
-            for oid, pool in six.iteritems(pool_status):
+            for oid, pool in pool_status.items():
                 if oid != listener_id:
                     pool_id = oid
                     pools = listener_dict['pools']
