@@ -21,8 +21,6 @@ from oslo_log import log as logging
 from octavia.amphorae.backends.health_daemon import status_message
 from octavia.common import exceptions
 from octavia.db import repositories
-from octavia.i18n import _LI
-
 
 UDP_MAX_SIZE = 64 * 1024
 LOG = logging.getLogger(__name__)
@@ -41,7 +39,7 @@ class UDPStatusGetter(object):
         self.ip = cfg.CONF.health_manager.bind_ip
         self.port = cfg.CONF.health_manager.bind_port
         self.sockaddr = None
-        LOG.info(_LI('attempting to listen on %(ip)s port %(port)s'),
+        LOG.info('attempting to listen on %(ip)s port %(port)s',
                  {'ip': self.ip, 'port': self.port})
         self.sock = None
         self.update(self.key, self.ip, self.port)
@@ -65,10 +63,10 @@ class UDPStatusGetter(object):
             if self.sock is not None:
                 self.sock.close()
             self.sock = socket.socket(ai_family, socket.SOCK_DGRAM)
-            self.sock.bind((ip, port))
+            self.sock.bind(self.sockaddr)
             if cfg.CONF.health_manager.sock_rlimit > 0:
                 rlimit = cfg.CONF.health_manager.sock_rlimit
-                LOG.info(_LI("setting sock rlimit to %s"), rlimit)
+                LOG.info("setting sock rlimit to %s", rlimit)
                 self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF,
                                      rlimit)
             break  # just used the first addr getaddrinfo finds
@@ -83,88 +81,90 @@ class UDPStatusGetter(object):
                  can be seen below. Note that listener_1 has no pools
                  and listener_4 has no members.
 
-        {"listeners": {
-        "listener_uuid_1": {
-            "pools": {},
-            "status": "OPEN",
-            "stats": {
-                "conns": 0,
-                "rx": 0,
-                "tx": 0
-            }
-        },
-        "listener_uuid_2": {
-            "pools": {
-                "pool_uuid_1": {
-                    "members": [
-                        {
-                            "member_uuid_1": "DOWN"
+        Example::
+
+            {
+              "listeners": {
+                "listener_uuid_1": {
+                  "pools": {},
+                  "status": "OPEN",
+                  "stats": {
+                    "conns": 0,
+                    "rx": 0,
+                    "tx": 0
+                  }
+                },
+                "listener_uuid_2": {
+                  "pools": {
+                    "pool_uuid_1": {
+                      "members": [{
+                          "member_uuid_1": "DOWN"
                         },
                         {
-                            "member_uuid_2": "DOWN"
+                          "member_uuid_2": "DOWN"
                         },
                         {
-                            "member_uuid_3": "DOWN"
+                          "member_uuid_3": "DOWN"
                         },
                         {
-                            "member_uuid_4": "DOWN"
+                          "member_uuid_4": "DOWN"
                         }
-                    ]
-                }
-            },
-            "status": "OPEN",
-            "stats": {
-                "conns": 0,
-                "rx": 0,
-                "tx": 0
-            }
-        },
-        "listener_uuid_3": {
-            "pools": {
-                "pool_uuid_2": {
-                    "members": [
-                        {
-                            "member_uuid_5": "DOWN"
+                      ]
+                    }
+                  },
+                  "status": "OPEN",
+                  "stats": {
+                    "conns": 0,
+                    "rx": 0,
+                    "tx": 0
+                  }
+                },
+                "listener_uuid_3": {
+                  "pools": {
+                    "pool_uuid_2": {
+                      "members": [{
+                          "member_uuid_5": "DOWN"
                         },
                         {
-                            "member_uuid_6": "DOWN"
+                          "member_uuid_6": "DOWN"
                         },
                         {
-                            "member_uuid_7": "DOWN"
+                          "member_uuid_7": "DOWN"
                         },
                         {
-                            "member_uuid_8": "DOWN"
+                          "member_uuid_8": "DOWN"
                         }
-                    ]
+                      ]
+                    }
+                  },
+                  "status": "OPEN",
+                  "stats": {
+                    "conns": 0,
+                    "rx": 0,
+                    "tx": 0
+                  }
+                },
+                "listener_uuid_4": {
+                  "pools": {
+                    "pool_uuid_3": {
+                      "members": []
+                    }
+                  },
+                  "status": "OPEN",
+                  "stats": {
+                    "conns": 0,
+                    "rx": 0,
+                    "tx": 0
+                  }
                 }
-            },
-            "status": "OPEN",
-            "stats": {
-                "conns": 0,
-                "rx": 0,
-                "tx": 0
+              },
+              "id": "amphora_uuid",
+              "seq": 1033
             }
-        },
-        "listener_uuid_4": {
-            "pools": {
-                "pool_uuid_3": {
-                    "members": []
-                }
-            },
-            "status": "OPEN",
-            "stats": {
-                "conns": 0,
-                "rx": 0,
-                "tx": 0
-            }
-        }
-    },
-    "id": "amphora_uuid",
-    "seq": 1033
-}
 
         """
         (data, srcaddr) = self.sock.recvfrom(UDP_MAX_SIZE)
+        LOG.debug('Received packet from %s', srcaddr)
         obj = status_message.unwrap_envelope(data, self.key)
         return obj, srcaddr
 
