@@ -29,8 +29,6 @@ from octavia.common import exceptions
 import octavia.common.validate as validate
 from octavia.db import api as db_api
 from octavia.db import prepare as db_prepare
-from octavia.i18n import _LI
-
 
 LOG = logging.getLogger(__name__)
 
@@ -51,12 +49,16 @@ class MembersController(base.BaseController):
         db_member = self._get_db_member(context.session, id)
         return self._convert_db_to_type(db_member, member_types.MemberResponse)
 
-    @wsme_pecan.wsexpose([member_types.MemberResponse])
+    @wsme_pecan.wsexpose([member_types.MemberResponse], ignore_extra_args=True)
     def get_all(self):
         """Lists all pool members of a pool."""
-        context = pecan.request.context.get('octavia_context')
-        db_members = self.repositories.member.get_all(
-            context.session, pool_id=self.pool_id)
+        pcontext = pecan.request.context
+        context = pcontext.get('octavia_context')
+
+        db_members, _ = self.repositories.member.get_all(
+            context.session,
+            pagination_helper=pcontext.get(constants.PAGINATION_HELPER),
+            pool_id=self.pool_id)
         return self._convert_db_to_type(db_members,
                                         [member_types.MemberResponse])
 
@@ -82,8 +84,8 @@ class MembersController(base.BaseController):
                 session, self.load_balancer_id,
                 constants.PENDING_UPDATE, constants.PENDING_UPDATE,
                 listener_ids=self._get_affected_listener_ids(session, member)):
-            LOG.info(_LI("Member cannot be created or modified because the "
-                         "Load Balancer is in an immutable state"))
+            LOG.info("Member cannot be created or modified because the "
+                     "Load Balancer is in an immutable state")
             lb_repo = self.repositories.load_balancer
             db_lb = lb_repo.get(session, id=self.load_balancer_id)
             raise exceptions.ImmutableObject(resource=db_lb._name(),
@@ -94,10 +96,13 @@ class MembersController(base.BaseController):
     def post(self, member):
         """Creates a pool member on a pool."""
         context = pecan.request.context.get('octavia_context')
+
+        member.project_id = self._get_lb_project_id(context.session,
+                                                    self.load_balancer_id)
+
         # Validate member subnet
-        if member.subnet_id and not validate.subnet_exists(member.subnet_id):
-            raise exceptions.NotFound(resource='Subnet',
-                                      id=member.subnet_id)
+        if member.subnet_id:
+            validate.subnet_exists(member.subnet_id)
 
         lock_session = db_api.get_session(autocommit=False)
         if self.repositories.check_quota_met(
@@ -130,7 +135,7 @@ class MembersController(base.BaseController):
                 lock_session.rollback()
 
         try:
-            LOG.info(_LI("Sending Creation of Member %s to handler"),
+            LOG.info("Sending Creation of Member %s to handler",
                      db_member.id)
             self.handler.create(db_member)
         except Exception:
@@ -153,7 +158,7 @@ class MembersController(base.BaseController):
         self._test_lb_and_listener_statuses(context.session, member=db_member)
 
         try:
-            LOG.info(_LI("Sending Update of Member %s to handler"), id)
+            LOG.info("Sending Update of Member %s to handler", id)
             self.handler.update(db_member, member)
         except Exception:
             with excutils.save_and_reraise_exception(reraise=False):
@@ -173,7 +178,7 @@ class MembersController(base.BaseController):
         self._test_lb_and_listener_statuses(context.session, member=db_member)
 
         try:
-            LOG.info(_LI("Sending Deletion of Member %s to handler"),
+            LOG.info("Sending Deletion of Member %s to handler",
                      db_member.id)
             self.handler.delete(db_member)
         except Exception:
