@@ -30,9 +30,26 @@ HEALTH_MONITOR_PING = 'PING'
 HEALTH_MONITOR_TCP = 'TCP'
 HEALTH_MONITOR_HTTP = 'HTTP'
 HEALTH_MONITOR_HTTPS = 'HTTPS'
+HEALTH_MONITOR_TLS_HELLO = 'TLS-HELLO'
 SUPPORTED_HEALTH_MONITOR_TYPES = (HEALTH_MONITOR_HTTP, HEALTH_MONITOR_HTTPS,
-                                  HEALTH_MONITOR_PING, HEALTH_MONITOR_TCP)
-HEALTH_MONITOR_HTTP_DEFAULT_METHOD = 'GET'
+                                  HEALTH_MONITOR_PING, HEALTH_MONITOR_TCP,
+                                  HEALTH_MONITOR_TLS_HELLO)
+HEALTH_MONITOR_HTTP_METHOD_GET = 'GET'
+HEALTH_MONITOR_HTTP_METHOD_HEAD = 'HEAD'
+HEALTH_MONITOR_HTTP_METHOD_POST = 'POST'
+HEALTH_MONITOR_HTTP_METHOD_PUT = 'PUT'
+HEALTH_MONITOR_HTTP_METHOD_DELETE = 'DELETE'
+HEALTH_MONITOR_HTTP_METHOD_TRACE = 'TRACE'
+HEALTH_MONITOR_HTTP_METHOD_OPTIONS = 'OPTIONS'
+HEALTH_MONITOR_HTTP_METHOD_CONNECT = 'CONNECT'
+HEALTH_MONITOR_HTTP_METHOD_PATCH = 'PATCH'
+HEALTH_MONITOR_HTTP_DEFAULT_METHOD = HEALTH_MONITOR_HTTP_METHOD_GET
+SUPPORTED_HEALTH_MONITOR_HTTP_METHODS = (
+    HEALTH_MONITOR_HTTP_METHOD_GET, HEALTH_MONITOR_HTTP_METHOD_HEAD,
+    HEALTH_MONITOR_HTTP_METHOD_POST, HEALTH_MONITOR_HTTP_METHOD_PUT,
+    HEALTH_MONITOR_HTTP_METHOD_DELETE, HEALTH_MONITOR_HTTP_METHOD_TRACE,
+    HEALTH_MONITOR_HTTP_METHOD_OPTIONS, HEALTH_MONITOR_HTTP_METHOD_CONNECT,
+    HEALTH_MONITOR_HTTP_METHOD_PATCH)
 HEALTH_MONITOR_DEFAULT_EXPECTED_CODES = '200'
 HEALTH_MONITOR_DEFAULT_URL_PATH = '/'
 
@@ -43,8 +60,21 @@ PROTOCOL_TCP = 'TCP'
 PROTOCOL_HTTP = 'HTTP'
 PROTOCOL_HTTPS = 'HTTPS'
 PROTOCOL_TERMINATED_HTTPS = 'TERMINATED_HTTPS'
+PROTOCOL_PROXY = 'PROXY'
 SUPPORTED_PROTOCOLS = (PROTOCOL_TCP, PROTOCOL_HTTPS, PROTOCOL_HTTP,
-                       PROTOCOL_TERMINATED_HTTPS)
+                       PROTOCOL_TERMINATED_HTTPS, PROTOCOL_PROXY)
+
+# API Integer Ranges
+MIN_PORT_NUMBER = 1
+MAX_PORT_NUMBER = 65535
+
+MIN_CONNECTION_LIMIT = -1
+
+MIN_WEIGHT = 0
+MAX_WEIGHT = 256
+
+MIN_HM_RETRIES = 1
+MAX_HM_RETRIES = 10
 
 # Note: The database Amphora table has a foreign key constraint against
 #       the provisioning_status table
@@ -75,9 +105,12 @@ ONLINE = 'ONLINE'
 OFFLINE = 'OFFLINE'
 DEGRADED = 'DEGRADED'
 ERROR = 'ERROR'
+DRAINING = 'DRAINING'
 NO_MONITOR = 'NO_MONITOR'
 OPERATING_STATUS = 'operating_status'
-SUPPORTED_OPERATING_STATUSES = (ONLINE, OFFLINE, DEGRADED, ERROR, NO_MONITOR)
+PROVISIONING_STATUS = 'provisioning_status'
+SUPPORTED_OPERATING_STATUSES = (ONLINE, OFFLINE, DEGRADED, ERROR, DRAINING,
+                                NO_MONITOR)
 
 AMPHORA_VM = 'VM'
 SUPPORTED_AMPHORA_TYPES = (AMPHORA_VM,)
@@ -110,6 +143,7 @@ SUPPORTED_L7POLICY_ACTIONS = (L7POLICY_ACTION_REJECT,
                               L7POLICY_ACTION_REDIRECT_TO_URL,
                               L7POLICY_ACTION_REDIRECT_TO_POOL)
 
+MIN_POLICY_POSITION = 1
 # Largest a 32-bit integer can be, which is a limitation
 # here if you're using MySQL, as most probably are. This just needs
 # to be larger than any existing rule position numbers which will
@@ -152,6 +186,7 @@ LOADBALANCER = 'loadbalancer'
 LOADBALANCER_ID = 'loadbalancer_id'
 SERVER_GROUP_ID = 'server_group_id'
 ANTI_AFFINITY = 'anti-affinity'
+SOFT_ANTI_AFFINITY = 'soft-anti-affinity'
 MEMBER = 'member'
 MEMBER_ID = 'member_id'
 COMPUTE_ID = 'compute_id'
@@ -258,6 +293,11 @@ SERVICE_AUTH = 'service_auth'
 
 RPC_NAMESPACE_CONTROLLER_AGENT = 'controller'
 
+# Build Type Priority
+LB_CREATE_FAILOVER_PRIORITY = 20
+LB_CREATE_NORMAL_PRIORITY = 40
+LB_CREATE_SPARES_POOL_PRIORITY = 60
+BUILD_TYPE_PRIORITY = 'build_type_priority'
 
 # Active standalone roles and topology
 TOPOLOGY_SINGLE = 'SINGLE'
@@ -307,12 +347,19 @@ DOWN = 'DOWN'
 HAPROXY_BACKEND_STATUSES = (UP, DOWN)
 
 
+DRAIN = 'DRAIN'
+MAINT = 'MAINT'
 NO_CHECK = 'no check'
 
-HAPROXY_MEMBER_STATUSES = (UP, DOWN, NO_CHECK)
+# DRAIN = member is weight 0 and is in draining mode
+# MAINT = member is downed for maintenance? not sure when this happens
+# NO_CHECK = no health monitor is enabled
+HAPROXY_MEMBER_STATUSES = (UP, DOWN, DRAIN, MAINT, NO_CHECK)
 
 # Quota Constants
 QUOTA_UNLIMITED = -1
+MIN_QUOTA = QUOTA_UNLIMITED
+MAX_QUOTA = 2000000000
 
 API_VERSION = '0.5'
 
@@ -323,6 +370,7 @@ KEEPALIVED_JINJA2_SYSVINIT = 'keepalived.sysvinit.j2'
 CHECK_SCRIPT_CONF = 'keepalived_check_script.conf.j2'
 
 PLUGGED_INTERFACES = '/var/lib/octavia/plugged_interfaces'
+HAPROXY_USER_GROUP_CFG = '/var/lib/octavia/haproxy-default-user-group.conf'
 AMPHORA_NAMESPACE = 'amphora-haproxy'
 
 # List of HTTP headers which are supported for insertion
@@ -366,8 +414,55 @@ KEEPALIVED_UPSTART = 'octavia-keepalived.conf'
 # Authentication
 KEYSTONE = 'keystone'
 NOAUTH = 'noauth'
+TESTING = 'testing'
 
+# Amphora distro-specific data
 UBUNTU_AMP_NET_DIR_TEMPLATE = '/etc/netns/{netns}/network/interfaces.d/'
 RH_AMP_NET_DIR_TEMPLATE = '/etc/netns/{netns}/sysconfig/network-scripts/'
 UBUNTU = 'ubuntu'
 CENTOS = 'centos'
+
+# Pagination, sorting, filtering values
+APPLICATION_JSON = 'application/json'
+PAGINATION_HELPER = 'pagination_helper'
+ASC = 'asc'
+DESC = 'desc'
+ALLOWED_SORT_DIR = (ASC, DESC)
+DEFAULT_SORT_DIR = ASC
+DEFAULT_SORT_KEYS = ['created_at', 'id']
+DEFAULT_PAGE_SIZE = 1000
+
+# RBAC
+LOADBALANCER_API = 'os_load-balancer_api'
+RULE_API_READ = 'rule:load-balancer:read'
+RULE_API_READ_GLOBAL = 'rule:load-balancer:read-global'
+RULE_API_WRITE = 'rule:load-balancer:write'
+RULE_API_READ_QUOTA = 'rule:load-balancer:read-quota'
+RULE_API_READ_QUOTA_GLOBAL = 'rule:load-balancer:read-quota-global'
+RULE_API_WRITE_QUOTA = 'rule:load-balancer:write-quota'
+RBAC_LOADBALANCER = '{}:loadbalancer:'.format(LOADBALANCER_API)
+RBAC_LISTENER = '{}:listener:'.format(LOADBALANCER_API)
+RBAC_POOL = '{}:pool:'.format(LOADBALANCER_API)
+RBAC_MEMBER = '{}:member:'.format(LOADBALANCER_API)
+RBAC_HEALTHMONITOR = '{}:healthmonitor:'.format(LOADBALANCER_API)
+RBAC_L7POLICY = '{}:l7policy:'.format(LOADBALANCER_API)
+RBAC_L7RULE = '{}:l7rule:'.format(LOADBALANCER_API)
+RBAC_QUOTA = '{}:quota:'.format(LOADBALANCER_API)
+RBAC_POST = 'post'
+RBAC_PUT = 'put'
+RBAC_DELETE = 'delete'
+RBAC_GET_ONE = 'get_one'
+RBAC_GET_ALL = 'get_all'
+RBAC_GET_ALL_GLOBAL = 'get_all-global'
+RBAC_GET_DEFAULTS = 'get_defaults'
+RBAC_GET_STATS = 'get_stats'
+RBAC_GET_STATUS = 'get_status'
+
+# PROVIDERS
+# TODO(johnsom) When providers are implemented, this should be removed.
+OCTAVIA = 'octavia'
+SUPPORTED_PROVIDERS = OCTAVIA,
+
+# FLAVORS
+# TODO(johnsom) When flavors are implemented, this should be removed.
+SUPPORTED_FLAVORS = ()
