@@ -18,27 +18,51 @@ Several handy validation functions that go beyond simple type checking.
 Defined here so these can also be used at deeper levels than the API.
 """
 
+
 import re
 
+import netaddr
+from oslo_config import cfg
 import rfc3986
 
 from octavia.common import constants
 from octavia.common import exceptions
+from octavia.common import utils
 
 
-def url(url):
+CONF = cfg.CONF
+
+
+def url(url, require_scheme=True):
     """Raises an error if the url doesn't look like a URL."""
     try:
-        if not rfc3986.is_valid_uri(url, require_scheme=True):
+        if not rfc3986.is_valid_uri(url, require_scheme=require_scheme):
             raise exceptions.InvalidURL(url=url)
         p_url = rfc3986.urlparse(rfc3986.normalize_uri(url))
-        if p_url.scheme != 'http' and p_url.scheme != 'https':
-            raise exceptions.InvalidURL(url=url)
+        if require_scheme:
+            if p_url.scheme != 'http' and p_url.scheme != 'https':
+                raise exceptions.InvalidURL(url=url)
     except Exception:
         raise exceptions.InvalidURL(url=url)
     return True
 
 
+def url_path(url_path):
+    """Raises an error if the url_path doesn't look like a URL Path."""
+    try:
+        p_url = rfc3986.urlparse(rfc3986.normalize_uri(url_path))
+
+        if (
+            p_url.scheme or p_url.userinfo or p_url.host or
+            p_url.port or
+            p_url.path is None
+        ):
+            raise exceptions.InvalidURLPath(url_path=url_path)
+    except Exception:
+        raise exceptions.InvalidURLPath(url_path=url_path)
+    return True
+
+
 def header_name(header, what=None):
     """Raises an error if header does not look like an HTML header name."""
     p = re.compile(constants.HTTP_HEADER_NAME_REGEX)
@@ -160,8 +184,8 @@ def sanitize_l7policy_api_args(l7policy, create=False):
             l7policy.update({'redirect_pool_id': None})
             l7policy.pop('redirect_pool', None)
         elif l7policy['action'] == constants.L7POLICY_ACTION_REDIRECT_TO_POOL:
-            if (not l7policy.get('redirect_pool_id')
-                    and not l7policy.get('redirect_pool')):
+            if (not l7policy.get('redirect_pool_id') and
+                    not l7policy.get('redirect_pool')):
                 raise exceptions.InvalidL7PolicyArgs(
                     msg='redirect_pool_id or redirect_pool must not be None')
             l7policy.update({'redirect_url': None})
@@ -198,3 +222,55 @@ def sanitize_l7policy_api_args(l7policy, create=False):
     if len(l7policy.keys()) == 0:
         raise exceptions.InvalidL7PolicyArgs(msg='Invalid update options')
     return l7policy
+
+
+def port_exists(port_id):
+    """Raises an exception when a port does not exist."""
+    network_driver = utils.get_network_driver()
+    try:
+        port = network_driver.get_port(port_id)
+    except Exception:
+        raise exceptions.InvalidSubresource(resource='Port', id=port_id)
+    return port
+
+
+def subnet_exists(subnet_id):
+    """Raises an exception when a subnet does not exist."""
+    network_driver = utils.get_network_driver()
+    try:
+        subnet = network_driver.get_subnet(subnet_id)
+    except Exception:
+        raise exceptions.InvalidSubresource(resource='Subnet', id=subnet_id)
+    return subnet
+
+
+def network_exists_optionally_contains_subnet(network_id, subnet_id=None):
+    """Raises an exception when a network does not exist.
+
+    If a subnet is provided, also validate the network contains that subnet.
+    """
+    network_driver = utils.get_network_driver()
+    try:
+        network = network_driver.get_network(network_id)
+    except Exception:
+        raise exceptions.InvalidSubresource(resource='Network', id=network_id)
+    if subnet_id:
+        if not network.subnets or subnet_id not in network.subnets:
+            raise exceptions.InvalidSubresource(resource='Subnet',
+                                                id=subnet_id)
+    return network
+
+
+def network_allowed_by_config(network_id):
+    if CONF.networking.valid_vip_networks:
+        valid_networks = map(str.lower, CONF.networking.valid_vip_networks)
+        if network_id not in valid_networks:
+            raise exceptions.ValidationException(detail=_(
+                'Supplied VIP network_id is not allowed by the configuration '
+                'of this deployment.'))
+
+
+def is_ip_member_of_cidr(address, cidr):
+    if netaddr.IPAddress(address) in netaddr.IPNetwork(cidr):
+        return True
+    return False
