@@ -49,18 +49,20 @@ class BaseNeutronDriver(base.AbstractNetworkDriver):
     def _check_extension_enabled(self, extension_alias):
         if extension_alias in self._check_extension_cache:
             status = self._check_extension_cache[extension_alias]
-            LOG.debug('Neutron extension {ext} cached as {status}'.format(
-                ext=extension_alias,
-                status='enabled' if status else 'disabled'))
+            LOG.debug('Neutron extension %(ext)s cached as %(status)s',
+                      {
+                          'ext': extension_alias,
+                          'status': 'enabled' if status else 'disabled'
+                      })
         else:
             try:
                 self.neutron_client.show_extension(extension_alias)
-                LOG.debug('Neutron extension {ext} found enabled'.format(
-                    ext=extension_alias))
+                LOG.debug('Neutron extension %(ext)s found enabled',
+                          {'ext': extension_alias})
                 self._check_extension_cache[extension_alias] = True
             except neutron_client_exceptions.NotFound:
-                LOG.debug('Neutron extension {ext} is not enabled'.format(
-                    ext=extension_alias))
+                LOG.debug('Neutron extension %(ext)s is not enabled',
+                          {'ext': extension_alias})
                 self._check_extension_cache[extension_alias] = False
         return self._check_extension_cache[extension_alias]
 
@@ -72,6 +74,7 @@ class BaseNeutronDriver(base.AbstractNetworkDriver):
                 break
         return data_models.Vip(ip_address=fixed_ip.ip_address,
                                subnet_id=fixed_ip.subnet_id,
+                               network_id=port.network_id,
                                port_id=port.id,
                                load_balancer=load_balancer,
                                load_balancer_id=load_balancer.id)
@@ -150,7 +153,7 @@ class BaseNeutronDriver(base.AbstractNetworkDriver):
                            resource_type)(resource)
         except neutron_client_exceptions.NotFound:
             message = _('{resource_type} not found '
-                        '({resource_type} id: {resource_id}.').format(
+                        '({resource_type} id: {resource_id}).').format(
                 resource_type=resource_type, resource_id=resource_id)
             raise getattr(base, '%sNotFound' %
                           resource_type.capitalize())(message)
