@@ -30,17 +30,17 @@ from oslo_log import log as logging
 import six
 from six.moves.urllib import error
 from six.moves.urllib import request as urllib2
+from tempest import clients
 from tempest.common import credentials_factory
 from tempest.common import waiters
 from tempest import config
-from tempest import exceptions
 from tempest.lib.common.utils import test_utils
 from tempest.lib import exceptions as lib_exc
-from tempest.scenario import manager
 from tempest import test
 
 
 from octavia.i18n import _
+from octavia.tests.tempest.common import manager
 from octavia.tests.tempest.v1.clients import health_monitors_client
 from octavia.tests.tempest.v1.clients import listeners_client
 from octavia.tests.tempest.v1.clients import load_balancers_client
@@ -74,7 +74,7 @@ class BaseTestCase(manager.NetworkScenarioTest):
 
         auth_provider = mgr.auth_provider
         region = config.network.region or config.identity.region
-        self.client_args = [auth_provider, 'octavia', region]
+        self.client_args = [auth_provider, 'load-balancer', region]
         self.load_balancers_client = (
             load_balancers_client.LoadBalancersClient(*self.client_args))
         self.listeners_client = (
@@ -91,10 +91,11 @@ class BaseTestCase(manager.NetworkScenarioTest):
         self._set_net_and_subnet()
 
         # admin network client needed for assigning octavia port to flip
-        admin_manager = credentials_factory.AdminManager()
-        admin_manager.auth_provider.fill_credentials()
-        self.floating_ips_client_admin = admin_manager.floating_ips_client
-        self.ports_client_admin = admin_manager.ports_client
+        os_admin = clients.Manager(
+            credentials_factory.get_configured_admin_credentials())
+        os_admin.auth_provider.fill_credentials()
+        self.floating_ips_client_admin = os_admin.floating_ips_client
+        self.ports_client_admin = os_admin.ports_client
 
     @classmethod
     def skip_checks(cls):
@@ -118,12 +119,14 @@ class BaseTestCase(manager.NetworkScenarioTest):
         """
         tenant_id = self.load_balancers_client.tenant_id
         try:
-            tenant_net = self._list_networks(tenant_id=tenant_id)[0]
+            tenant_net = self.os_admin.networks_client.list_networks(
+                tenant_id=tenant_id)['networks'][0]
         except IndexError:
             tenant_net = None
 
         if tenant_net:
-            tenant_subnet = self._list_subnets(tenant_id=tenant_id)[0]
+            tenant_subnet = self.os_admin.subnets_client.list_subnets(
+                tenant_id=tenant_id)['subnets'][0]
             self.subnet = tenant_subnet
             self.network = tenant_net
         else:
@@ -133,7 +136,8 @@ class BaseTestCase(manager.NetworkScenarioTest):
             # with the fixed network is the one we want.  In the future, we
             # should instead pull a subnet id from config, which is set by
             # devstack/admin/etc.
-            subnet = self._list_subnets(network_id=self.network['id'])[0]
+            subnet = self.os_admin.subnets_client.list_subnets(
+                network_id=self.network['id'])['subnets'][0]
             self.subnet = subnet
 
     def _create_security_group_for_test(self):
@@ -184,8 +188,8 @@ class BaseTestCase(manager.NetworkScenarioTest):
         server = server['server']
         self.servers_keypairs[server['id']] = keypair
 
-        LOG.info(('servers_keypairs looks like this(format): {0}'.format(
-            self.servers_keypairs)))
+        LOG.info('servers_keypairs looks like this(format): %s',
+                 self.servers_keypairs)
 
         if (config.network.public_network_id and not
                 config.network.project_networks_reachable):
@@ -214,11 +218,11 @@ class BaseTestCase(manager.NetworkScenarioTest):
     def _stop_server(self, name):
         for sname, value in six.iteritems(self.servers):
             if sname == name:
-                LOG.info(('STOPPING SERVER: {0}'.format(sname)))
+                LOG.info('STOPPING SERVER: %s', sname)
                 self.servers_client.stop_server(value)
                 waiters.wait_for_server_status(self.servers_client,
                                                value, 'SHUTOFF')
-        LOG.info(('STOPPING SERVER COMPLETED!'))
+        LOG.info('STOPPING SERVER COMPLETED!')
 
     def _start_server(self, name):
         for sname, value in six.iteritems(self.servers):
@@ -258,7 +262,7 @@ class BaseTestCase(manager.NetworkScenarioTest):
                 private_key=private_key)
 
             with tempfile.NamedTemporaryFile() as key:
-                key.write(private_key)
+                key.write(private_key.encode('utf-8'))
                 key.flush()
                 self.copy_file_to_host(httpd,
                                        "/dev/shm/httpd",
@@ -285,8 +289,8 @@ class BaseTestCase(manager.NetworkScenarioTest):
         self.assertTrue(self.listener)
         self.addCleanup(self._cleanup_listener, self.listener['id'],
                         load_balancer_id)
-        LOG.info(('Waiting for lb status on create listener id: {0}'.format(
-            self.listener['id'])))
+        LOG.info('Waiting for lb status on create listener id: %s',
+                 self.listener['id'])
         self._wait_for_load_balancer_status(load_balancer_id)
 
         return self.listener
@@ -332,8 +336,8 @@ class BaseTestCase(manager.NetworkScenarioTest):
                                                   **create_pool_kwargs)
         self.assertTrue(self.pool)
         self.addCleanup(self._cleanup_pool, self.pool['id'], load_balancer_id)
-        LOG.info(('Waiting for lb status on create pool id: {0}'.format(
-            self.pool['id'])))
+        LOG.info('Waiting for lb status on create pool id: %s',
+                 self.pool['id'])
         self._wait_for_load_balancer_status(load_balancer_id)
         return self.pool
 
@@ -342,6 +346,11 @@ class BaseTestCase(manager.NetworkScenarioTest):
             self.load_balancers_client.delete_load_balancer, load_balancer_id)
         self._wait_for_load_balancer_status(load_balancer_id, delete=True)
 
+    def _delete_load_balancer_cascade(self, load_balancer_id):
+        self.load_balancers_client.delete_load_balancer_cascade(
+            load_balancer_id)
+        self._wait_for_load_balancer_status(load_balancer_id, delete=True)
+
     def _cleanup_listener(self, listener_id, load_balancer_id=None):
         test_utils.call_and_ignore_notfound_exc(
             self.listeners_client.delete_listener, load_balancer_id,
@@ -384,7 +393,7 @@ class BaseTestCase(manager.NetworkScenarioTest):
                     lb_id=load_balancer_id,
                     pool_id=pool_id,
                     **create_member_kwargs)
-                LOG.info(('Waiting for lb status on create member...'))
+                LOG.info('Waiting for lb status on create member...')
                 self._wait_for_load_balancer_status(load_balancer_id)
                 self.members.append(member)
             self.assertTrue(self.members)
@@ -392,8 +401,7 @@ class BaseTestCase(manager.NetworkScenarioTest):
     def _assign_floating_ip_to_lb_vip(self, lb):
         public_network_id = config.network.public_network_id
 
-        LOG.info(('assign_floating_ip_to_lb_vip  lb: {0} type: {1}'.format(
-            lb, type(lb))))
+        LOG.info('assign_floating_ip_to_lb_vip  lb: %s type: %s', lb, type(lb))
         port_id = lb['vip']['port_id']
 
         floating_ip = self._create_floating_ip(
@@ -416,7 +424,7 @@ class BaseTestCase(manager.NetworkScenarioTest):
 
         :param dict floating_ip: floating IP dict to check status
         :param status: target status
-        :raises: AssertionError if status doesn't match
+        :raises AssertionError: if status doesn't match
         """
 
         # TODO(ptoohill): Find a way to utilze the proper client method
@@ -428,7 +436,7 @@ class BaseTestCase(manager.NetworkScenarioTest):
                       show_floatingip(floatingip_id)['floatingip'])
             return status == result['status']
 
-        test.call_until_true(refresh, 100, 1)
+        test_utils.call_until_true(refresh, 100, 1)
 
         floating_ip = self.floating_ips_client_admin.show_floatingip(
             floatingip_id)['floatingip']
@@ -437,8 +445,8 @@ class BaseTestCase(manager.NetworkScenarioTest):
                                  "failed  to reach status: {st}"
                          .format(fp=floating_ip, cst=floating_ip['status'],
                                  st=status))
-        LOG.info("FloatingIP: {fp} is at status: {st}"
-                 .format(fp=floating_ip, st=status))
+        LOG.info('FloatingIP: %(fp)s is at status: %(st)s',
+                 {'fp': floating_ip, 'st': status})
 
     def _create_load_balancer(self, ip_version=4, persistence_type=None):
         """Create a load balancer.
@@ -455,8 +463,7 @@ class BaseTestCase(manager.NetworkScenarioTest):
             **self.create_lb_kwargs)
         lb_id = self.load_balancer['id']
         self.addCleanup(self._cleanup_load_balancer, lb_id)
-        LOG.info(('Waiting for lb status on create load balancer id: {0}'
-                  .format(lb_id)))
+        LOG.info('Waiting for lb status on create load balancer id: %s', lb_id)
         self.load_balancer = self._wait_for_load_balancer_status(
             load_balancer_id=lb_id,
             provisioning_status='ACTIVE',
@@ -506,8 +513,8 @@ class BaseTestCase(manager.NetworkScenarioTest):
         lb_client.create_load_balancer_over_quota(
             **self.create_lb_kwargs)
 
-        LOG.info(('Waiting for lb status on create load balancer id: {0}'
-                  .format(lb_id)))
+        LOG.info('Waiting for lb status on create load balancer id: %s',
+                 lb_id)
         self.load_balancer = self._wait_for_load_balancer_status(
             load_balancer_id=lb_id,
             provisioning_status='ACTIVE',
@@ -530,9 +537,9 @@ class BaseTestCase(manager.NetworkScenarioTest):
                 else:
                     raise e
 
-            LOG.info(('provisioning_status: {0}  operating_status: {1}'.format(
-                lb.get('provisioning_status'),
-                lb.get('operating_status'))))
+            LOG.info('provisioning_status: %s operating_status: %s',
+                     lb.get('provisioning_status'),
+                     lb.get('operating_status'))
 
             if delete and lb.get('provisioning_status') == 'DELETED':
                 break
@@ -591,13 +598,15 @@ class BaseTestCase(manager.NetworkScenarioTest):
         3. Check that no unexpected members were balanced.
         """
         members = members or ['server1_0', 'server1_1']
+        members = list(map(
+            lambda x: six.b(x) if type(x) == six.text_type else x, members))
         LOG.info(_('Checking all members are balanced...'))
         self._wait_for_http_service(self.vip_ip)
-        LOG.info(_('Connection to {vip} is valid').format(vip=self.vip_ip))
+        LOG.info(_('Connection to %(vip)s is valid'), {'vip': self.vip_ip})
         counters = self._send_concurrent_requests(self.vip_ip)
         for member, counter in six.iteritems(counters):
-            LOG.info(_('Member {member} saw {counter} requests.').format(
-                member=member, counter=counter))
+            LOG.info(_('Member %(member)s saw %(counter)s requests.'),
+                     {'member': member, 'counter': counter})
             self.assertGreater(counter, 0,
                                'Member %s never balanced' % member)
         for member in members:
@@ -615,18 +624,18 @@ class BaseTestCase(manager.NetworkScenarioTest):
     def _wait_for_http_service(self, check_ip, port=80):
         def try_connect(check_ip, port):
             try:
-                LOG.info(('checking connection to ip: {0} port: {1}'.format(
-                    check_ip, port)))
+                LOG.info('checking connection to ip: %s port: %d',
+                         check_ip, port)
                 resp = urllib2.urlopen("http://{0}:{1}/".format(check_ip,
                                                                 port))
                 if resp.getcode() == 200:
                     return True
                 return False
             except IOError as e:
-                LOG.info(('Got IOError in check connection: {0}'.format(e)))
+                LOG.info('Got IOError in check connection: %s', e)
                 return False
             except error.HTTPError as e:
-                LOG.info(('Got HTTPError in check connection: {0}'.format(e)))
+                LOG.info('Got HTTPError in check connection: %s', e)
                 return False
 
         timeout = config.validation.ping_timeout
@@ -634,7 +643,7 @@ class BaseTestCase(manager.NetworkScenarioTest):
         while not try_connect(check_ip, port):
             if (time.time() - start) > timeout:
                 message = "Timed out trying to connect to %s" % check_ip
-                raise exceptions.TimeoutException(message)
+                raise lib_exc.TimeoutException(message)
             time.sleep(1)
 
     def _send_requests(self, vip_ip, path=''):
@@ -649,8 +658,9 @@ class BaseTestCase(manager.NetworkScenarioTest):
                     counters[server] += 1
             # HTTP exception means fail of server, so don't increase counter
             # of success and continue connection tries
-            except (error.HTTPError, error.URLError, socket.timeout) as e:
-                LOG.info(('Got Error in sending request: {0}'.format(e)))
+            except (error.HTTPError, error.URLError,
+                    socket.timeout, socket.error) as e:
+                LOG.info('Got Error in sending request: %s', e)
                 continue
         return counters
 
@@ -688,7 +698,7 @@ class BaseTestCase(manager.NetworkScenarioTest):
         for ct in client_threads:
             timeout -= ct.join(timeout)
             if timeout <= 0:
-                LOG.error("Client thread {0} timed out".format(ct.name))
+                LOG.error('Client thread %s timed out', ct.name)
                 return dict()
             for server in list(ct.counters):
                 if server not in total_counters:
@@ -696,19 +706,6 @@ class BaseTestCase(manager.NetworkScenarioTest):
                 total_counters[server] += ct.counters[server]
         return total_counters
 
-    def _traffic_validation_after_stopping_server(self):
-        """Check that the requests are sent to the only ACTIVE server."""
-
-        LOG.info(('Starting traffic_validation_after_stopping_server...'))
-        counters = self._send_requests(self.vip_ip, ["server1", "server2"])
-        LOG.info(('Counters is: {0}'.format(counters)))
-
-        # Assert that no traffic is sent to server1.
-        for member, counter in six.iteritems(counters):
-            if member == 'server1':
-                self.assertEqual(counter, 0,
-                                 'Member %s is not balanced' % member)
-
     def _check_load_balancing_after_deleting_resources(self):
         """Check load balancer after deleting resources
 
@@ -818,16 +815,15 @@ class BaseTestCase(manager.NetworkScenarioTest):
         return self.execute(cmd)
 
     def execute(self, cmd, cwd=None):
-        args = shlex.split(cmd.encode('utf-8'))
+        args = shlex.split(cmd)
         subprocess_args = {'stdout': subprocess.PIPE,
                            'stderr': subprocess.STDOUT,
                            'cwd': cwd}
         proc = subprocess.Popen(args, **subprocess_args)
         stdout, stderr = proc.communicate()
         if proc.returncode != 0:
-            LOG.error(("Command {0} returned with exit status {1},"
-                      "output {2}, error {3}").format(cmd, proc.returncode,
-                                                      stdout, stderr))
+            LOG.error('Command %s returned with exit status %s,output %s, '
+                      'error %s', cmd, proc.returncode, stdout, stderr)
         return stdout
 
     def _set_quotas(self, project_id=None, load_balancer=20, listener=20,
@@ -839,7 +835,7 @@ class BaseTestCase(manager.NetworkScenarioTest):
             'pool': pool, 'health_monitor': health_monitor, 'member': member}}
         return self.quotas_client.update_quotas(project_id, **body)
 
-    def _create_load_balancer_tree(self, ip_version=4):
+    def _create_load_balancer_tree(self, ip_version=4, cleanup=True):
         # TODO(ptoohill): remove or null out project ID when Octavia supports
         # keystone auth and automatically populates it for us.
         project_id = self.networks_client.tenant_id
@@ -863,9 +859,10 @@ class BaseTestCase(manager.NetworkScenarioTest):
                               .create_load_balancer_graph(create_lb))
 
         load_balancer_id = self.load_balancer['id']
-        self.addCleanup(self._cleanup_load_balancer, load_balancer_id)
-        LOG.info(('Waiting for lb status on create load balancer id: {0}'
-                  .format(load_balancer_id)))
+        if cleanup:
+            self.addCleanup(self._cleanup_load_balancer, load_balancer_id)
+        LOG.info('Waiting for lb status on create load balancer id: %s',
+                 load_balancer_id)
         self.load_balancer = self._wait_for_load_balancer_status(
             load_balancer_id)
 
