@@ -30,22 +30,24 @@ from oslo_log import log as logging
 import six
 from six.moves.urllib import error
 from six.moves.urllib import request as urllib2
+from tempest import clients
 from tempest.common import credentials_factory
 from tempest.common import waiters
 from tempest import config
 from tempest import exceptions
 from tempest.lib.common.utils import test_utils
 from tempest.lib import exceptions as lib_exc
-from tempest.scenario import manager
 from tempest import test
 
 
 from octavia.i18n import _
+from octavia.tests.tempest.common import manager
 from octavia.tests.tempest.v1.clients import health_monitors_client
 from octavia.tests.tempest.v1.clients import listeners_client
 from octavia.tests.tempest.v1.clients import load_balancers_client
 from octavia.tests.tempest.v1.clients import members_client
 from octavia.tests.tempest.v1.clients import pools_client
+from octavia.tests.tempest.v2.clients import quotas_client
 
 config = config.CONF
 
@@ -76,7 +78,7 @@ class BaseTestCase(manager.NetworkScenarioTest):
 
         auth_provider = mgr.auth_provider
         region = config.network.region or config.identity.region
-        self.client_args = [auth_provider, 'octavia', region]
+        self.client_args = [auth_provider, 'load-balancer', region]
         self.load_balancers_client = (
             load_balancers_client.LoadBalancersClient(*self.client_args))
         self.listeners_client = (
@@ -87,12 +89,14 @@ class BaseTestCase(manager.NetworkScenarioTest):
         self.health_monitors_client = (
             health_monitors_client.HealthMonitorsClient(
                 *self.client_args))
+        self.quotas_client = quotas_client.QuotasClient(*self.client_args)
 
         # admin network client needed for assigning octavia port to flip
-        admin_manager = credentials_factory.AdminManager()
-        admin_manager.auth_provider.fill_credentials()
-        self.floating_ips_client_admin = admin_manager.floating_ips_client
-        self.ports_client_admin = admin_manager.ports_client
+        os_admin = clients.Manager(
+            credentials_factory.get_configured_admin_credentials())
+        os_admin.auth_provider.fill_credentials()
+        self.floating_ips_client_admin = os_admin.floating_ips_client
+        self.ports_client_admin = os_admin.ports_client
 
     @classmethod
     def skip_checks(cls):
@@ -115,12 +119,14 @@ class BaseTestCase(manager.NetworkScenarioTest):
         fallback in absence of tenant networking.
         """
         try:
-            tenant_net = self._list_networks(tenant_id=self.tenant_id)[0]
+            tenant_net = self.os_admin.networks_client.list_networks(
+                tenant_id=self.tenant_id)['networks'][0]
         except IndexError:
             tenant_net = None
 
         if tenant_net:
-            tenant_subnet = self._list_subnets(tenant_id=self.tenant_id)[0]
+            tenant_subnet = self.os_admin.subnets_client.list_subnets(
+                tenant_id=self.tenant_id)['subnets'][0]
             self.subnet = tenant_subnet
             self.network = tenant_net
         else:
@@ -130,7 +136,8 @@ class BaseTestCase(manager.NetworkScenarioTest):
             # with the fixed network is the one we want.  In the future, we
             # should instead pull a subnet id from config, which is set by
             # devstack/admin/etc.
-            subnet = self._list_subnets(network_id=self.network['id'])[0]
+            subnet = self.os_admin.subnets_client.list_subnets(
+                network_id=self.network['id'])['subnets'][0]
             self.subnet = subnet
 
     def _create_security_group_for_test(self):
@@ -180,8 +187,8 @@ class BaseTestCase(manager.NetworkScenarioTest):
         server = server['server']
         self.servers_keypairs[server['id']] = keypair
 
-        LOG.info(('servers_keypairs looks like this(format): {0}'.format(
-            self.servers_keypairs)))
+        LOG.info('servers_keypairs looks like this(format): %s',
+                 self.servers_keypairs)
 
         if (config.network.public_network_id and not
                 config.network.project_networks_reachable):
@@ -210,11 +217,11 @@ class BaseTestCase(manager.NetworkScenarioTest):
     def _stop_server(self, name):
         for sname, value in six.iteritems(self.servers):
             if sname == name:
-                LOG.info(('STOPPING SERVER: {0}'.format(sname)))
+                LOG.info('STOPPING SERVER: %s', sname)
                 self.servers_client.stop_server(value)
                 waiters.wait_for_server_status(self.servers_client,
                                                value, 'SHUTOFF')
-        LOG.info(('STOPPING SERVER COMPLETED!'))
+        LOG.info('STOPPING SERVER COMPLETED!')
 
     def _start_server(self, name):
         for sname, value in six.iteritems(self.servers):
@@ -281,8 +288,8 @@ class BaseTestCase(manager.NetworkScenarioTest):
         self.assertTrue(self.listener)
         self.addCleanup(self._cleanup_listener, self.listener['id'],
                         load_balancer_id)
-        LOG.info(('Waiting for lb status on create listener id: {0}'.format(
-            self.listener['id'])))
+        LOG.info('Waiting for lb status on create listener id: %s',
+                 self.listener['id'])
         self._wait_for_load_balancer_status(load_balancer_id)
 
         return self.listener
@@ -328,8 +335,8 @@ class BaseTestCase(manager.NetworkScenarioTest):
                                                   **create_pool_kwargs)
         self.assertTrue(self.pool)
         self.addCleanup(self._cleanup_pool, self.pool['id'], load_balancer_id)
-        LOG.info(('Waiting for lb status on create pool id: {0}'.format(
-            self.pool['id'])))
+        LOG.info('Waiting for lb status on create pool id: %s',
+                 self.pool['id'])
         self._wait_for_load_balancer_status(load_balancer_id)
         return self.pool
 
@@ -380,7 +387,7 @@ class BaseTestCase(manager.NetworkScenarioTest):
                     lb_id=load_balancer_id,
                     pool_id=pool_id,
                     **create_member_kwargs)
-                LOG.info(('Waiting for lb status on create member...'))
+                LOG.info('Waiting for lb status on create member...')
                 self._wait_for_load_balancer_status(load_balancer_id)
                 self.members.append(member)
             self.assertTrue(self.members)
@@ -388,8 +395,7 @@ class BaseTestCase(manager.NetworkScenarioTest):
     def _assign_floating_ip_to_lb_vip(self, lb):
         public_network_id = config.network.public_network_id
 
-        LOG.info(('assign_floating_ip_to_lb_vip  lb: {0} type: {1}'.format(
-            lb, type(lb))))
+        LOG.info('assign_floating_ip_to_lb_vip  lb: %s type: %s', lb, type(lb))
         port_id = lb['vip']['port_id']
 
         floating_ip = self._create_floating_ip(
@@ -412,7 +418,7 @@ class BaseTestCase(manager.NetworkScenarioTest):
 
         :param dict floating_ip: floating IP dict to check status
         :param status: target status
-        :raises: AssertionError if status doesn't match
+        :raises AssertionError: if status doesn't match
         """
 
         # TODO(ptoohill): Find a way to utilze the proper client method
@@ -424,7 +430,7 @@ class BaseTestCase(manager.NetworkScenarioTest):
                       show_floatingip(floatingip_id)['floatingip'])
             return status == result['status']
 
-        test.call_until_true(refresh, 100, 1)
+        test_utils.call_until_true(refresh, 100, 1)
 
         floating_ip = self.floating_ips_client_admin.show_floatingip(
             floatingip_id)['floatingip']
@@ -433,8 +439,8 @@ class BaseTestCase(manager.NetworkScenarioTest):
                                  "failed  to reach status: {st}"
                          .format(fp=floating_ip, cst=floating_ip['status'],
                                  st=status))
-        LOG.info("FloatingIP: {fp} is at status: {st}"
-                 .format(fp=floating_ip, st=status))
+        LOG.info('FloatingIP: %(fp)s is at status: %(st)s',
+                 {'fp': floating_ip, 'st': status})
 
     def _create_load_balancer(self, ip_version=4, persistence_type=None):
         """Create a load balancer.
@@ -449,8 +455,7 @@ class BaseTestCase(manager.NetworkScenarioTest):
             **self.create_lb_kwargs)
         lb_id = self.load_balancer['id']
         self.addCleanup(self._cleanup_load_balancer, lb_id)
-        LOG.info(('Waiting for lb status on create load balancer id: {0}'
-                  .format(lb_id)))
+        LOG.info('Waiting for lb status on create load balancer id: %s', lb_id)
         self.load_balancer = self._wait_for_load_balancer_status(
             load_balancer_id=lb_id,
             provisioning_status='ACTIVE',
@@ -477,6 +482,35 @@ class BaseTestCase(manager.NetworkScenarioTest):
             security_groups=[self.security_group['id']])
         return lb_id
 
+    def _create_load_balancer_over_quota(self):
+        """Attempt to create a load balancer over quota.
+
+        Creates two load balancers one after the other expecting
+        the second create to exceed the configured quota.
+
+        :returns: Response body from the request
+        """
+        self.create_lb_kwargs = {
+            'vip': {'subnet_id': self.subnet['id']},
+            'project_id': self.load_balancers_client.tenant_id}
+        self.load_balancer = self.load_balancers_client.create_load_balancer(
+            **self.create_lb_kwargs)
+        lb_id = self.load_balancer['id']
+        self.addCleanup(self._cleanup_load_balancer, lb_id)
+
+        self.create_lb_kwargs = {
+            'vip': {'subnet_id': self.subnet['id']},
+            'project_id': self.load_balancers_client.tenant_id}
+        lb_client = self.load_balancers_client
+        lb_client.create_load_balancer_over_quota(
+            **self.create_lb_kwargs)
+
+        LOG.info('Waiting for lb status on create load balancer id: %s', lb_id)
+        self.load_balancer = self._wait_for_load_balancer_status(
+            load_balancer_id=lb_id,
+            provisioning_status='ACTIVE',
+            operating_status='ONLINE')
+
     def _wait_for_load_balancer_status(self, load_balancer_id,
                                        provisioning_status='ACTIVE',
                                        operating_status='ONLINE',
@@ -494,9 +528,9 @@ class BaseTestCase(manager.NetworkScenarioTest):
                 else:
                     raise e
 
-            LOG.info(('provisioning_status: {0}  operating_status: {1}'.format(
-                lb.get('provisioning_status'),
-                lb.get('operating_status'))))
+            LOG.info('provisioning_status: %s  operating_status: %s',
+                     lb.get('provisioning_status'),
+                     lb.get('operating_status'))
 
             if delete and lb.get('provisioning_status') == 'DELETED':
                 break
@@ -557,11 +591,11 @@ class BaseTestCase(manager.NetworkScenarioTest):
         members = members or ['server1_0', 'server1_1']
         LOG.info(_('Checking all members are balanced...'))
         self._wait_for_http_service(self.vip_ip)
-        LOG.info(_('Connection to {vip} is valid').format(vip=self.vip_ip))
+        LOG.info(_('Connection to %(vip)s is valid'), {'vip': self.vip_ip})
         counters = self._send_concurrent_requests(self.vip_ip)
         for member, counter in six.iteritems(counters):
-            LOG.info(_('Member {member} saw {counter} requests.').format(
-                member=member, counter=counter))
+            LOG.info(_('Member %(member)s saw %(counter)s requests.'),
+                     {'member': member, 'counter': counter})
             self.assertGreater(counter, 0,
                                'Member %s never balanced' % member)
         for member in members:
@@ -579,18 +613,18 @@ class BaseTestCase(manager.NetworkScenarioTest):
     def _wait_for_http_service(self, check_ip, port=80):
         def try_connect(check_ip, port):
             try:
-                LOG.info(('checking connection to ip: {0} port: {1}'.format(
-                    check_ip, port)))
+                LOG.info('checking connection to ip: %s port: %s',
+                         check_ip, port)
                 resp = urllib2.urlopen("http://{0}:{1}/".format(check_ip,
                                                                 port))
                 if resp.getcode() == 200:
                     return True
                 return False
             except IOError as e:
-                LOG.info(('Got IOError in check connection: {0}'.format(e)))
+                LOG.info('Got IOError in check connection: %s', e)
                 return False
             except error.HTTPError as e:
-                LOG.info(('Got HTTPError in check connection: {0}'.format(e)))
+                LOG.info('Got HTTPError in check connection: %s', e)
                 return False
 
         timeout = config.validation.ping_timeout
@@ -613,8 +647,9 @@ class BaseTestCase(manager.NetworkScenarioTest):
                     counters[server] += 1
             # HTTP exception means fail of server, so don't increase counter
             # of success and continue connection tries
-            except (error.HTTPError, error.URLError, socket.timeout) as e:
-                LOG.info(('Got Error in sending request: {0}'.format(e)))
+            except (error.HTTPError, error.URLError,
+                    socket.timeout, socket.error) as e:
+                LOG.info('Got Error in sending request: %s', e)
                 continue
         return counters
 
@@ -652,7 +687,7 @@ class BaseTestCase(manager.NetworkScenarioTest):
         for ct in client_threads:
             timeout -= ct.join(timeout)
             if timeout <= 0:
-                LOG.error("Client thread {0} timed out".format(ct.name))
+                LOG.error("Client thread %s timed out", ct.name)
                 return dict()
             for server in list(ct.counters):
                 if server not in total_counters:
@@ -663,9 +698,9 @@ class BaseTestCase(manager.NetworkScenarioTest):
     def _traffic_validation_after_stopping_server(self):
         """Check that the requests are sent to the only ACTIVE server."""
 
-        LOG.info(('Starting traffic_validation_after_stopping_server...'))
+        LOG.info('Starting traffic_validation_after_stopping_server...')
         counters = self._send_requests(self.vip_ip, ["server1", "server2"])
-        LOG.info(('Counters is: {0}'.format(counters)))
+        LOG.info('Counters is: %s', counters)
 
         # Assert that no traffic is sent to server1.
         for member, counter in six.iteritems(counters):
@@ -789,7 +824,80 @@ class BaseTestCase(manager.NetworkScenarioTest):
         proc = subprocess.Popen(args, **subprocess_args)
         stdout, stderr = proc.communicate()
         if proc.returncode != 0:
-            LOG.error(("Command {0} returned with exit status {1},"
-                      "output {2}, error {3}").format(cmd, proc.returncode,
-                                                      stdout, stderr))
+            LOG.error('Command %s returned with exit status %s, output %s, '
+                      'error %s', cmd, proc.returncode, stdout, stderr)
         return stdout
+
+    def _set_quotas(self, project_id=None, load_balancer=20, listener=20,
+                    pool=20, health_monitor=20, member=20):
+        if not project_id:
+            project_id = self.networks_client.tenant_id
+        body = {'quota': {
+            'load_balancer': load_balancer, 'listener': listener,
+            'pool': pool, 'health_monitor': health_monitor, 'member': member}}
+        return self.quotas_client.update_quotas(project_id, **body)
+
+    def _create_load_balancer_tree(self, ip_version=4, cleanup=True):
+        # TODO(ptoohill): remove or null out project ID when Octavia supports
+        # keystone auth and automatically populates it for us.
+        project_id = self.networks_client.tenant_id
+
+        create_members = self._create_members_kwargs(self.subnet['id'])
+        create_pool = {'project_id': project_id,
+                       'lb_algorithm': 'ROUND_ROBIN',
+                       'protocol': 'HTTP',
+                       'members': create_members}
+        create_listener = {'project_id': project_id,
+                           'protocol': 'HTTP',
+                           'protocol_port': 80,
+                           'default_pool': create_pool}
+        create_lb = {'project_id': project_id,
+                     'vip': {'subnet_id': self.subnet['id']},
+                     'listeners': [create_listener]}
+
+        # Set quotas back and finish the test
+        self._set_quotas(project_id=project_id)
+        self.load_balancer = (self.load_balancers_client
+                              .create_load_balancer_graph(create_lb))
+
+        load_balancer_id = self.load_balancer['id']
+        if cleanup:
+            self.addCleanup(self._cleanup_load_balancer, load_balancer_id)
+        LOG.info('Waiting for lb status on create load balancer id: %s',
+                 load_balancer_id)
+        self.load_balancer = self._wait_for_load_balancer_status(
+            load_balancer_id)
+
+        self.vip_ip = self.load_balancer['vip'].get('ip_address')
+
+        # if the ipv4 is used for lb, then fetch the right values from
+        # tempest.conf file
+        if ip_version == 4:
+            if (config.network.public_network_id and
+                    not config.network.project_networks_reachable):
+                load_balancer = self.load_balancer
+                self._assign_floating_ip_to_lb_vip(load_balancer)
+                self.vip_ip = self.floating_ips[
+                    load_balancer['id']][0]['floating_ip_address']
+
+        # Currently the ovs-agent is not enforcing security groups on the
+        # vip port - see https://bugs.launchpad.net/neutron/+bug/1163569
+        # However the linuxbridge-agent does, and it is necessary to add a
+        # security group with a rule that allows tcp port 80 to the vip port.
+        self.ports_client_admin.update_port(
+            self.load_balancer['vip']['port_id'],
+            security_groups=[self.security_group['id']])
+
+    def _create_members_kwargs(self, subnet_id=None):
+        """Create one or more Members
+
+        In case there is only one server, create both members with the same ip
+        but with different ports to listen on.
+        """
+        create_member_kwargs = []
+        for server_id, ip in six.iteritems(self.server_fixed_ips):
+            create_member_kwargs.append({'ip_address': ip,
+                                         'protocol_port': 80,
+                                         'weight': 50,
+                                         'subnet_id': subnet_id})
+        return create_member_kwargs
