@@ -37,6 +37,7 @@ class AgentJinjaTestCase(base.TestCase):
         self.conf.config(group="haproxy_amphora",
                          base_cert_dir='/var/lib/octavia/certs')
         self.conf.config(group="haproxy_amphora", use_upstart='True')
+        self.conf.config(group="haproxy_amphora", user_group='nogroup')
         self.conf.config(group="haproxy_amphora", base_path='/var/lib/octavia')
         self.conf.config(group="haproxy_amphora", bind_host='0.0.0.0')
         self.conf.config(group="haproxy_amphora", bind_port=9443)
@@ -64,7 +65,8 @@ class AgentJinjaTestCase(base.TestCase):
                            'haproxy_cmd = /usr/sbin/haproxy\n'
                            'respawn_count = 2\n'
                            'respawn_interval = 2\n'
-                           'use_upstart = True\n\n'
+                           'use_upstart = True\n'
+                           'user_group = nogroup\n\n'
                            '[health_manager]\n'
                            'controller_ip_port_list = 192.0.2.10:5555\n'
                            'heartbeat_interval = 10\n'
@@ -76,6 +78,7 @@ class AgentJinjaTestCase(base.TestCase):
                            '/etc/octavia/certs/server.pem\n'
                            'agent_server_network_dir = '
                            '/etc/network/interfaces.d/\n'
+                           'agent_request_read_timeout = 120\n'
                            'amphora_id = ' + AMP_ID)
         agent_cfg = ajc.build_agent_config(AMP_ID)
         self.assertEqual(expected_config, agent_cfg)
@@ -95,7 +98,8 @@ class AgentJinjaTestCase(base.TestCase):
                            'haproxy_cmd = /usr/sbin/haproxy\n'
                            'respawn_count = 2\n'
                            'respawn_interval = 2\n'
-                           'use_upstart = False\n\n'
+                           'use_upstart = False\n'
+                           'user_group = nogroup\n\n'
                            '[health_manager]\n'
                            'controller_ip_port_list = 192.0.2.10:5555\n'
                            'heartbeat_interval = 10\n'
@@ -109,6 +113,7 @@ class AgentJinjaTestCase(base.TestCase):
                            '/etc/network/interfaces.d/\n'
                            'agent_server_network_file = '
                            '/etc/network/interfaces\n'
+                           'agent_request_read_timeout = 120\n'
                            'amphora_id = ' + AMP_ID)
         agent_cfg = ajc.build_agent_config(AMP_ID)
         self.assertEqual(expected_config, agent_cfg)
