@@ -91,9 +91,11 @@ class TestNovaClient(base.TestCase):
 
     def setUp(self):
         conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
+        self.conf = conf
         self.net_name = "lb-mgmt-net"
-        conf.config(group="networking", lb_network_name=self.net_name)
-        conf.config(group="controller_worker", amp_boot_network_list=[1, 2])
+        conf.config(group="controller_worker",
+                    amp_boot_network_list=['1', '2'])
+        self.conf = conf
 
         self.amphora = models.Amphora(
             compute_id=uuidutils.generate_uuid(),
@@ -104,9 +106,10 @@ class TestNovaClient(base.TestCase):
         self.nova_response = mock.Mock()
         self.nova_response.id = self.amphora.compute_id
         self.nova_response.status = 'ACTIVE'
+        self.nova_response.fault = 'FAKE_FAULT'
 
         self.interface_list = mock.MagicMock()
-        self.interface_list.net_id = 1
+        self.interface_list.net_id = '1'
         self.interface_list.fixed_ips = [mock.MagicMock()]
         self.interface_list.fixed_ips[0] = {'ip_address': '10.0.0.1'}
 
@@ -161,7 +164,73 @@ class TestNovaClient(base.TestCase):
             files='Files Blah',
             userdata='Blah',
             config_drive=True,
-            scheduler_hints=None)
+            scheduler_hints=None,
+            availability_zone=None
+        )
+
+    def test_build_with_availability_zone(self):
+        FAKE_AZ = "my_availability_zone"
+        self.conf.config(group="nova", availability_zone=FAKE_AZ)
+
+        amphora_id = self.manager.build(amphora_flavor=1, image_id=1,
+                                        key_name=1,
+                                        sec_groups=1,
+                                        network_ids=[1],
+                                        port_ids=[2],
+                                        user_data='Blah',
+                                        config_drive_files='Files Blah')
+
+        self.assertEqual(self.amphora.compute_id, amphora_id)
+
+        self.manager.manager.create.assert_called_with(
+            name="amphora_name",
+            nics=[{'net-id': 1}, {'port-id': 2}],
+            image=1,
+            flavor=1,
+            key_name=1,
+            security_groups=1,
+            files='Files Blah',
+            userdata='Blah',
+            config_drive=True,
+            scheduler_hints=None,
+            availability_zone=FAKE_AZ
+        )
+
+    def test_build_with_random_amphora_name_length(self):
+        self.conf.config(group="nova", random_amphora_name_length=15)
+        self.addCleanup(self.conf.config,
+                        group='nova', random_amphora_name_length=0)
+
+        self.manager.build(name="b" * 50, image_id=1)
+        self.assertEqual(
+            15, len(self.manager.manager.create.call_args[1]['name']))
+
+    def test_build_with_default_boot_network(self):
+        self.conf.config(group="controller_worker",
+                         amp_boot_network_list='')
+        amphora_id = self.manager.build(amphora_flavor=1, image_id=1,
+                                        key_name=1,
+                                        sec_groups=1,
+                                        network_ids=None,
+                                        port_ids=[2],
+                                        user_data='Blah',
+                                        config_drive_files='Files Blah')
+
+        self.assertEqual(self.amphora.compute_id, amphora_id)
+
+        self.manager.manager.create.assert_called_with(
+            name="amphora_name",
+            nics=[{'port-id': 2}],
+            image=1,
+            flavor=1,
+            key_name=1,
+            security_groups=1,
+            files='Files Blah',
+            userdata='Blah',
+            config_drive=True,
+            scheduler_hints=None,
+            availability_zone=None
+        )
 
     def test_bad_build(self):
         self.manager.manager.create.side_effect = Exception
@@ -201,8 +270,9 @@ class TestNovaClient(base.TestCase):
                           self.manager.status, self.amphora.id)
 
     def test_get_amphora(self):
-        amphora = self.manager.get_amphora(self.amphora.compute_id)
+        amphora, fault = self.manager.get_amphora(self.amphora.compute_id)
         self.assertEqual(self.amphora, amphora)
+        self.assertEqual(self.nova_response.fault, fault)
         self.manager.manager.get.called_with(server=amphora.id)
 
     def test_bad_get_amphora(self):
@@ -211,15 +281,16 @@ class TestNovaClient(base.TestCase):
                           self.manager.get_amphora, self.amphora.id)
 
     def test_translate_amphora(self):
-        amphora = self.manager._translate_amphora(self.nova_response)
+        amphora, fault = self.manager._translate_amphora(self.nova_response)
         self.assertEqual(self.amphora, amphora)
+        self.assertEqual(self.nova_response.fault, fault)
         self.nova_response.interface_list.called_with()
 
     def test_bad_translate_amphora(self):
         self.nova_response.interface_list.side_effect = Exception
         self.manager._nova_client.networks.get.side_effect = Exception
-        self.assertIsNone(
-            self.manager._translate_amphora(self.nova_response).lb_network_ip)
+        amphora, fault = self.manager._translate_amphora(self.nova_response)
+        self.assertIsNone(amphora.lb_network_ip)
         self.nova_response.interface_list.called_with()
 
     def test_create_server_group(self):
