@@ -86,25 +86,21 @@ class TestControllerWorker(base.TestCase):
 
         self.conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
 
-        _health_mon_mock.pool.load_balancer.amphorae = _amphora_mock
-        _health_mon_mock.pool.listeners = [_listener_mock]
-        _health_mon_mock.pool.load_balancer = _load_balancer_mock
-        _health_mon_mock.pool.load_balancer.vip = _vip_mock
-        _listener_mock.load_balancer = _load_balancer_mock
-        _listener_mock.load_balancer.amphorae = _amphora_mock
-        _listener_mock.load_balancer.vip = _vip_mock
-        _member_mock.pool.listeners = [_listener_mock]
-        _member_mock.pool.load_balancer = _load_balancer_mock
-        _member_mock.pool.load_balancer.vip = _vip_mock
         _pool_mock.listeners = [_listener_mock]
         _pool_mock.load_balancer = _load_balancer_mock
-        _pool_mock.load_balancer.vip = _vip_mock
+        _health_mon_mock.pool = _pool_mock
+        self.amp_mock = _amphora_mock
+        _load_balancer_mock.amphorae = _amphora_mock
+        _load_balancer_mock.vip = _vip_mock
+        _listener_mock.load_balancer = _load_balancer_mock
+        _member_mock.pool = _pool_mock
         _l7policy_mock.listener = _listener_mock
         _l7rule_mock.l7policy = _l7policy_mock
 
         fetch_mock = mock.MagicMock(return_value=AMP_ID)
         _flow_mock.storage.fetch = fetch_mock
 
+        _pool_mock.id = POOL_ID
         _health_mon_mock.pool_id = POOL_ID
         _health_mon_mock.id = HM_ID
 
@@ -133,7 +129,10 @@ class TestControllerWorker(base.TestCase):
         amp = cw.create_amphora()
 
         (base_taskflow.BaseTaskFlowEngine._taskflow_load.
-            assert_called_once_with('TEST'))
+            assert_called_once_with(
+                'TEST',
+                store={constants.BUILD_TYPE_PRIORITY:
+                       constants.LB_CREATE_SPARES_POOL_PRIORITY}))
 
         _flow_mock.run.assert_called_once_with()
 
@@ -203,7 +202,9 @@ class TestControllerWorker(base.TestCase):
                                            constants.LISTENERS:
                                                [_listener_mock],
                                            constants.LOADBALANCER:
-                                               _load_balancer_mock}))
+                                               _load_balancer_mock,
+                                           constants.POOL:
+                                               _pool_mock}))
 
         _flow_mock.run.assert_called_once_with()
 
@@ -234,11 +235,12 @@ class TestControllerWorker(base.TestCase):
             assert_called_once_with(_flow_mock,
                                     store={constants.HEALTH_MON:
                                            _health_mon_mock,
-                                           constants.POOL_ID: HM_ID,
                                            constants.LISTENERS:
                                                [_listener_mock],
                                            constants.LOADBALANCER:
-                                               _load_balancer_mock}))
+                                               _load_balancer_mock,
+                                           constants.POOL:
+                                               _pool_mock}))
 
         _flow_mock.run.assert_called_once_with()
 
@@ -270,6 +272,8 @@ class TestControllerWorker(base.TestCase):
             assert_called_once_with(_flow_mock,
                                     store={constants.HEALTH_MON:
                                            _health_mon_mock,
+                                           constants.POOL:
+                                               _pool_mock,
                                            constants.LISTENERS:
                                                [_listener_mock],
                                            constants.LOADBALANCER:
@@ -400,7 +404,8 @@ class TestControllerWorker(base.TestCase):
         mock_taskflow_load.return_value = mock_eng
         store = {
             constants.LOADBALANCER_ID: LB_ID,
-            'update_dict': {'topology': constants.TOPOLOGY_SINGLE}
+            'update_dict': {'topology': constants.TOPOLOGY_SINGLE},
+            constants.BUILD_TYPE_PRIORITY: constants.LB_CREATE_NORMAL_PRIORITY
         }
         setattr(mock_lb_repo_get.return_value, 'listeners', [])
 
@@ -441,7 +446,8 @@ class TestControllerWorker(base.TestCase):
         mock_taskflow_load.return_value = mock_eng
         store = {
             constants.LOADBALANCER_ID: LB_ID,
-            'update_dict': {'topology': constants.TOPOLOGY_ACTIVE_STANDBY}
+            'update_dict': {'topology': constants.TOPOLOGY_ACTIVE_STANDBY},
+            constants.BUILD_TYPE_PRIORITY: constants.LB_CREATE_NORMAL_PRIORITY
         }
         setattr(mock_lb_repo_get.return_value, 'listeners', [])
 
@@ -483,7 +489,8 @@ class TestControllerWorker(base.TestCase):
         mock_taskflow_load.return_value = mock_eng
         store = {
             constants.LOADBALANCER_ID: LB_ID,
-            'update_dict': {'topology': constants.TOPOLOGY_SINGLE}
+            'update_dict': {'topology': constants.TOPOLOGY_SINGLE},
+            constants.BUILD_TYPE_PRIORITY: constants.LB_CREATE_NORMAL_PRIORITY
         }
 
         cw = controller_worker.ControllerWorker()
@@ -532,7 +539,8 @@ class TestControllerWorker(base.TestCase):
         mock_taskflow_load.return_value = mock_eng
         store = {
             constants.LOADBALANCER_ID: LB_ID,
-            'update_dict': {'topology': constants.TOPOLOGY_ACTIVE_STANDBY}
+            'update_dict': {'topology': constants.TOPOLOGY_ACTIVE_STANDBY},
+            constants.BUILD_TYPE_PRIORITY: constants.LB_CREATE_NORMAL_PRIORITY
         }
 
         cw = controller_worker.ControllerWorker()
@@ -626,7 +634,7 @@ class TestControllerWorker(base.TestCase):
                 'LoadBalancerFlows.get_update_load_balancer_flow',
                 return_value=_flow_mock)
     @mock.patch('octavia.db.repositories.ListenerRepository.get_all',
-                return_value=[_listener_mock])
+                return_value=([_listener_mock], None))
     def test_update_load_balancer(self,
                                   mock_listener_repo_get_all,
                                   mock_get_update_lb_flow,
@@ -690,7 +698,9 @@ class TestControllerWorker(base.TestCase):
                                            constants.LISTENERS:
                                                [_listener_mock],
                                            constants.LOADBALANCER:
-                                               _load_balancer_mock}))
+                                               _load_balancer_mock,
+                                           constants.POOL:
+                                               _pool_mock}))
 
         _flow_mock.run.assert_called_once_with()
 
@@ -722,7 +732,9 @@ class TestControllerWorker(base.TestCase):
                                    constants.LISTENERS:
                                        [_listener_mock],
                                    constants.LOADBALANCER:
-                                       _load_balancer_mock}))
+                                       _load_balancer_mock,
+                                   constants.POOL:
+                                       _pool_mock}))
 
         _flow_mock.run.assert_called_once_with()
 
@@ -755,6 +767,8 @@ class TestControllerWorker(base.TestCase):
                                                [_listener_mock],
                                            constants.LOADBALANCER:
                                                _load_balancer_mock,
+                                           constants.POOL:
+                                               _pool_mock,
                                            constants.UPDATE_DICT:
                                                MEMBER_UPDATE_DICT}))
 
@@ -1081,7 +1095,75 @@ class TestControllerWorker(base.TestCase):
                 _flow_mock,
                 store={constants.FAILED_AMPHORA: _amphora_mock,
                        constants.LOADBALANCER_ID:
-                           _amphora_mock.load_balancer_id}))
+                           _amphora_mock.load_balancer_id,
+                       constants.BUILD_TYPE_PRIORITY:
+                           constants.LB_CREATE_FAILOVER_PRIORITY
+                       }))
+
+        _flow_mock.run.assert_called_once_with()
+
+    @mock.patch('octavia.db.repositories.AmphoraHealthRepository.update')
+    def test_failover_deleted_amphora(self,
+                                      mock_update,
+                                      mock_api_get_session,
+                                      mock_dyn_log_listener,
+                                      mock_taskflow_load,
+                                      mock_pool_repo_get,
+                                      mock_member_repo_get,
+                                      mock_l7rule_repo_get,
+                                      mock_l7policy_repo_get,
+                                      mock_listener_repo_get,
+                                      mock_lb_repo_get,
+                                      mock_health_mon_repo_get,
+                                      mock_amp_repo_get):
+        mock_taskflow_load.reset_mock()
+        self.amp_mock.id = AMP_ID
+        self.amp_mock.status = constants.DELETED
+
+        cw = controller_worker.ControllerWorker()
+        cw.failover_amphora(AMP_ID)
+
+        mock_update.assert_called_with('TEST', AMP_ID, busy=True)
+        mock_taskflow_load.assert_not_called()
+
+    @mock.patch('octavia.controller.worker.flows.'
+                'amphora_flows.AmphoraFlows.get_failover_flow',
+                return_value=_flow_mock)
+    @mock.patch(
+        'octavia.db.repositories.AmphoraRepository.get_all_lbs_on_amphora',
+        return_value=[_load_balancer_mock])
+    def test_failover_amphora_anti_affinity(self,
+                                            mock_get_update_listener_flow,
+                                            mock_get_all_lbs_for_amp_mock,
+                                            mock_api_get_session,
+                                            mock_dyn_log_listener,
+                                            mock_taskflow_load,
+                                            mock_pool_repo_get,
+                                            mock_member_repo_get,
+                                            mock_l7rule_repo_get,
+                                            mock_l7policy_repo_get,
+                                            mock_listener_repo_get,
+                                            mock_lb_repo_get,
+                                            mock_health_mon_repo_get,
+                                            mock_amp_repo_get):
+
+        self.conf.config(group="nova", enable_anti_affinity=True)
+        _flow_mock.reset_mock()
+        _load_balancer_mock.server_group_id = "123"
+
+        cw = controller_worker.ControllerWorker()
+        cw.failover_amphora(AMP_ID)
+
+        (base_taskflow.BaseTaskFlowEngine._taskflow_load.
+            assert_called_once_with(
+                _flow_mock,
+                store={constants.FAILED_AMPHORA: _amphora_mock,
+                       constants.LOADBALANCER_ID:
+                           _amphora_mock.load_balancer_id,
+                       constants.BUILD_TYPE_PRIORITY:
+                           constants.LB_CREATE_FAILOVER_PRIORITY,
+                       constants.SERVER_GROUP_ID: "123",
+                       }))
 
         _flow_mock.run.assert_called_once_with()
 
